from jinjafy.cache.simplecache import cache_update_str as cache_update_str
from jinjafy.cache.simplecache import cache_contains_str as cache_contains_str

from jinjafy.cache.simplecache import cache_update_file as cache_update_file
from jinjafy.cache.simplecache import cache_contains_file as cache_contains_file

from jinjafy.cache.simplecache import cache_update_dir as cache_update_dir
from jinjafy.cache.simplecache import cache_contains_dir as cache_contains_dir